@@ -26,7 +26,10 @@ DocSearchProvider.prototype = {
             return null;
         return { 'id': resultId,
                  'name': docInfo.name,
-                 'icon': docInfo.createIcon(Search.RESULT_ICON_SIZE)};
+                 'createIcon': function(size) {
+                                   return docInfo.createIcon(size);
+                               }
+               };
     },
 
     activateResult: function(id, params) {
@@ -43,9 +46,5 @@ DocSearchProvider.prototype = {
 
     getSubsearchResultSet: function(previousResults, terms) {
         return this._docManager.subsearch(previousResults, terms);
-    },
-
-    expandSearch: function(terms) {
-        log('TODO expand docs search');
     }
 };
