@@ -1,5 +1,6 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
+const Clutter = imports.gi.Clutter;
 const DBus = imports.dbus;
 const GLib = imports.gi.GLib;
 const Lang = imports.lang;
@@ -93,16 +94,6 @@ NotificationDaemon.prototype = {
     _init: function() {
         DBus.session.exportObject('/org/freedesktop/Notifications', this);
 
-        this._everAcquiredName = false;
-        DBus.session.acquire_name('org.freedesktop.Notifications',
-                                  // We pass MANY_INSTANCES so that if
-                                  // notification-daemon is running, we'll
-                                  // get queued behind it and then get the
-                                  // name after killing it below
-                                  DBus.MANY_INSTANCES,
-                                  Lang.bind(this, this._acquiredName),
-                                  Lang.bind(this, this._lostName));
-
         this._sources = {};
         this._senderToPid = {};
         this._notifications = {};
@@ -117,22 +108,6 @@ NotificationDaemon.prototype = {
             Lang.bind(this, this._onFocusAppChanged));
     },
 
-    _acquiredName: function() {
-        this._everAcquiredName = true;
-    },
-
-    _lostName: function() {
-        if (this._everAcquiredName)
-            log('Lost name org.freedesktop.Notifications!');
-        else if (GLib.getenv('GNOME_SHELL_NO_REPLACE'))
-            log('Failed to acquire org.freedesktop.Notifications');
-        else {
-            log('Failed to acquire org.freedesktop.Notifications; trying again');
-            Util.killall('notification-daemon');
-            Util.killall('notify-osd');
-        }
-    },
-
     _iconForNotificationData: function(icon, hints, size) {
         let textureCache = St.TextureCache.get_default();
 
@@ -371,7 +346,7 @@ NotificationDaemon.prototype = {
         notification.setTransient(hints['transient'] == true);
 
         let sourceIconActor = source.useNotificationIcon ? this._iconForNotificationData(icon, hints, source.ICON_SIZE) : null;
-        source.notify(notification, sourceIconActor);
+        source.processNotification(notification, sourceIconActor);
     },
 
     CloseNotification: function(id) {
@@ -415,8 +390,7 @@ NotificationDaemon.prototype = {
         for (let id in this._sources) {
             let source = this._sources[id];
             if (source.app == tracker.focus_app) {
-                if (source.notification && !source.notification.resident)
-                    source.notification.destroy();
+                source.destroyNonResidentNotifications();
                 return;
             }
         }
@@ -461,20 +435,49 @@ Source.prototype = {
         MessageTray.Source.prototype._init.call(this, title);
 
         this._pid = pid;
+        this._appStateChangedId = 0;
         this._setApp();
         if (this.app)
             this.title = this.app.get_name();
         else
             this.useNotificationIcon = true;
-        this._isTrayIcon = false;
+        this._trayIcon = null;
     },
 
-    notify: function(notification, icon) {
+    processNotification: function(notification, icon) {
         if (!this.app)
             this._setApp();
         if (!this.app && icon)
             this._setSummaryIcon(icon);
-        MessageTray.Source.prototype.notify.call(this, notification);
+
+        let tracker = Shell.WindowTracker.get_default();
+        if (notification.resident && this.app && tracker.focus_app == this.app)
+            this.pushNotification(notification);
+        else
+            this.notify(notification);
+    },
+
+    handleSummaryClick: function() {
+        if (!this._trayIcon)
+            return false;
+
+        let event = Clutter.get_current_event();
+        if (event.type() != Clutter.EventType.BUTTON_RELEASE)
+            return false;
+
+        if (Main.overview.visible) {
+            // We can't just connect to Main.overview's 'hidden' signal,
+            // because it's emitted *before* it calls popModal()...
+            let id = global.connect('notify::stage-input-mode', Lang.bind(this,
+                function () {
+                    global.disconnect(id);
+                    this._trayIcon.click(event);
+                }));
+            Main.overview.hide();
+        } else {
+            this._trayIcon.click(event);
+        }
+        return true;
     },
 
     _setApp: function() {
@@ -485,9 +488,13 @@ Source.prototype = {
         if (!this.app)
             return;
 
+        // We only update the app if this.app is null, so we can't disconnect the old this._appStateChangedId
+        // even if it were non-zero for some reason.
+        this._appStateChangedId = this.app.connect('notify::state', Lang.bind(this,  this._appStateChanged));
+
         // Only override the icon if we were previously using
         // notification-based icons (ie, not a trayicon) or if it was unset before
-        if (!this._isTrayIcon) {
+        if (!this._trayIcon) {
             this.useNotificationIcon = false;
             this._setSummaryIcon(this.app.create_icon_texture (this.ICON_SIZE));
         }
@@ -496,15 +503,26 @@ Source.prototype = {
     setTrayIcon: function(icon) {
         this._setSummaryIcon(icon);
         this.useNotificationIcon = false;
-        this._isTrayIcon = true;
+        this._trayIcon = icon;
     },
 
-    _notificationClicked: function(notification) {
+    open: function(notification) {
+        this.destroyNonResidentNotifications();
         this.openApp();
     },
 
-    _notificationRemoved: function() {
-        if (!this._isTrayIcon)
+    _lastNotificationRemoved: function() {
+        if (!this._trayIcon)
+            this.destroy();
+    },
+
+    _appStateChanged: function() {
+        // Destroy notification sources when their apps exit.
+        // The app exiting would normally result in a tray icon being removed,
+        // so it should be ok to destroy the source associated with a tray icon
+        // here too, however we just let that happen through the code path
+        // associated with the tray icon being removed.
+        if (!this._isTrayIcon && this.app.get_state() == Shell.AppState.STOPPED)
             this.destroy();
     },
 
@@ -517,5 +535,13 @@ Source.prototype = {
             let mostRecentWindow = windows[0];
             Main.activateWindow(mostRecentWindow);
         }
+    },
+
+    destroy: function() {
+        if (this.app && this._appStateChangedId) {
+            this.app.disconnect(this._appStateChangedId);
+            this._appStateChangedId = 0;
+        }
+        MessageTray.Source.prototype.destroy.call(this);
     }
 };
