@@ -24,6 +24,7 @@ const PlaceDisplay = imports.ui.placeDisplay;
 const Tweener = imports.ui.tweener;
 const ViewSelector = imports.ui.viewSelector;
 const WorkspacesView = imports.ui.workspacesView;
+const WorkspaceThumbnail = imports.ui.workspaceThumbnail;
 
 // Time for initial animation going into Overview mode
 const ANIMATION_TIME = 0.25;
@@ -74,11 +75,13 @@ ShellInfo.prototype = {
             Main.messageTray.add(this._source);
         }
 
-        let notification = this._source.notification;
-        if (notification == null)
+        let notification = null;
+        if (this._source.notifications.length == 0) {
             notification = new MessageTray.Notification(this._source, text, null);
-        else
+        } else {
+            notification = this._source.notifications[0];
             notification.update(text, null, { clear: true });
+        }
 
         notification.setTransient(true);
 
@@ -180,10 +183,10 @@ Overview.prototype = {
         this._group.add_actor(this.viewSelector.actor);
 
         this._workspacesDisplay = new WorkspacesView.WorkspacesDisplay();
-        this.viewSelector.addViewTab(_("Windows"), this._workspacesDisplay.actor, 'text-x-generic');
+        this.viewSelector.addViewTab('windows', _("Windows"), this._workspacesDisplay.actor, 'text-x-generic');
 
         let appView = new AppDisplay.AllAppDisplay();
-        this.viewSelector.addViewTab(_("Applications"), appView.actor, 'system-run');
+        this.viewSelector.addViewTab('applications', _("Applications"), appView.actor, 'system-run');
 
         // Default search providers
         this.viewSelector.addSearchProvider(new AppDisplay.AppSearchProvider());
@@ -220,6 +223,7 @@ Overview.prototype = {
         this._resetWindowSwitchTimeout();
         this._lastHoveredWindow = null;
         DND.removeMonitor(this._dragMonitor);
+        this.endItemDrag();
     },
 
     _resetWindowSwitchTimeout: function() {
@@ -242,7 +246,8 @@ Overview.prototype = {
     _onDragMotion: function(dragEvent) {
         let targetIsWindow = dragEvent.targetActor &&
                              dragEvent.targetActor._delegate &&
-                             dragEvent.targetActor._delegate.metaWindow;
+                             dragEvent.targetActor._delegate.metaWindow &&
+                             !(dragEvent.targetActor._delegate instanceof WorkspaceThumbnail.WindowClone);
 
         this._windowSwitchTimestamp = global.get_current_time();
 
@@ -438,7 +443,7 @@ Overview.prototype = {
         let primary = global.get_primary_monitor();
         let rtl = (St.Widget.get_default_direction () == St.TextDirection.RTL);
 
-        let contentY = Panel.PANEL_HEIGHT;
+        let contentY = Main.panel.actor.height;
         let contentHeight = primary.height - contentY - Main.messageTray.actor.height;
 
         this._group.set_position(primary.x, primary.y);
@@ -473,6 +478,10 @@ Overview.prototype = {
         this.emit('item-drag-begin');
     },
 
+    cancelledItemDrag: function(source) {
+        this.emit('item-drag-cancelled');
+    },
+
     endItemDrag: function(source) {
         this.emit('item-drag-end');
     },
@@ -481,35 +490,14 @@ Overview.prototype = {
         this.emit('window-drag-begin');
     },
 
+    cancelledWindowDrag: function(source) {
+        this.emit('window-drag-cancelled');
+    },
+
     endWindowDrag: function(source) {
         this.emit('window-drag-end');
     },
 
-    // Returns the scale the Overview has when we just start zooming out
-    // to overview mode. That is, when just the active workspace is showing.
-    getZoomedInScale : function() {
-        return 1 / this.workspaces.getScale();
-    },
-
-    // Returns the position the Overview has when we just start zooming out
-    // to overview mode. That is, when just the active workspace is showing.
-    getZoomedInPosition : function() {
-        let [posX, posY] = this.workspaces.getActiveWorkspacePosition();
-        let scale = this.getZoomedInScale();
-
-        return [- posX * scale, - posY * scale];
-    },
-
-    // Returns the current scale of the Overview.
-    getScale : function() {
-        return this.workspaces.actor.scaleX;
-    },
-
-    // Returns the current position of the Overview.
-    getPosition : function() {
-        return [this.workspaces.actor.x, this.workspaces.actor.y];
-    },
-
     // show:
     //
     // Animates the overview visible and grabs mouse and keyboard input
@@ -563,30 +551,13 @@ Overview.prototype = {
                              });
         }
 
-        // Create a zoom out effect. First scale the workspaces view up and
-        // position it so that the active workspace fills up the whole screen,
-        // then transform it to its normal dimensions and position.
-        // The opposite transition is used in hide().
-        this.workspaces.actor.scaleX = this.workspaces.actor.scaleY = this.getZoomedInScale();
-        [this.workspaces.actor.x, this.workspaces.actor.y] = this.getZoomedInPosition();
-        let primary = global.get_primary_monitor();
-        Tweener.addTween(this.workspaces.actor,
-                         { x: primary.x - this._group.x,
-                           y: primary.y - this._group.y,
-                           scaleX: 1,
-                           scaleY: 1,
-                           transition: 'easeOutQuad',
-                           time: ANIMATION_TIME,
-                           onComplete: this._showDone,
-                           onCompleteScope: this
-                          });
-
-        // Make the other elements fade in.
         this._group.opacity = 0;
         Tweener.addTween(this._group,
                          { opacity: 255,
                            transition: 'easeOutQuad',
-                           time: ANIMATION_TIME
+                           time: ANIMATION_TIME,
+                           onComplete: this._showDone,
+                           onCompleteScope: this
                          });
 
         this._coverPane.raise_top();
@@ -698,27 +669,13 @@ Overview.prototype = {
 
         this.workspaces.hide();
 
-        // Create a zoom in effect by transforming the workspaces view so that
-        // the active workspace fills up the whole screen. The opposite
-        // transition is used in show().
-        let scale = this.getZoomedInScale();
-        let [posX, posY] = this.getZoomedInPosition();
-        Tweener.addTween(this.workspaces.actor,
-                         { x: posX,
-                           y: posY,
-                           scaleX: scale,
-                           scaleY: scale,
-                           transition: 'easeOutQuad',
-                           time: ANIMATION_TIME,
-                           onComplete: this._hideDone,
-                           onCompleteScope: this
-                          });
-
         // Make other elements fade out.
         Tweener.addTween(this._group,
                          { opacity: 0,
                            transition: 'easeOutQuad',
-                           time: ANIMATION_TIME
+                           time: ANIMATION_TIME,
+                           onComplete: this._hideDone,
+                           onCompleteScope: this
                          });
 
         this._coverPane.raise_top();
@@ -737,6 +694,7 @@ Overview.prototype = {
             this._animateNotVisible();
 
         this._syncInputMode();
+        global.sync_pointer();
     },
 
     _hideDone: function() {
