@@ -173,6 +173,8 @@ PopupBaseMenuItem.prototype = {
         cr.fill();
     },
 
+    // This returns column widths in logical order (i.e. from the dot
+    // to the image), not in visual order (left to right)
     getColumnWidths: function() {
         let widths = [];
         for (let i = 0, col = 0; i < this._children.length; i++) {
@@ -224,19 +226,36 @@ PopupBaseMenuItem.prototype = {
 
     _allocate: function(actor, box, flags) {
         let height = box.y2 - box.y1;
+        let direction = this.actor.get_direction();
 
         if (this._dot) {
+            // The dot is placed outside box
+            // one quarter of padding from the border of the container
+            // (so 3/4 from the inner border)
+            // (padding is box.x1)
             let dotBox = new Clutter.ActorBox();
             let dotWidth = Math.round(box.x1 / 2);
 
-            dotBox.x1 = Math.round(box.x1 / 4);
-            dotBox.x2 = dotBox.x1 + dotWidth;
+            if (direction == St.TextDirection.LTR) {
+                dotBox.x1 = Math.round(box.x1 / 4);
+                dotBox.x2 = dotBox.x1 + dotWidth;
+            } else {
+                dotBox.x2 = box.x2 + 3 * Math.round(box.x1 / 4);
+                dotBox.x1 = dotBox.x2 - dotWidth;
+            }
             dotBox.y1 = Math.round(box.y1 + (height - dotWidth) / 2);
             dotBox.y2 = dotBox.y1 + dotWidth;
             this._dot.allocate(dotBox, flags);
         }
 
-        let x = box.x1;
+        let x;
+        if (direction == St.TextDirection.LTR)
+            x = box.x1;
+        else
+            x = box.x2;
+        // if direction is ltr, x is the right edge of the last added
+        // actor, and it's constantly increasing, whereas if rtl, x is
+        // the left edge and it decreases
         for (let i = 0, col = 0; i < this._children.length; i++) {
             let child = this._children[i];
             let childBox = new Clutter.ActorBox();
@@ -244,9 +263,12 @@ PopupBaseMenuItem.prototype = {
             let [minWidth, naturalWidth] = child.actor.get_preferred_width(-1);
             let availWidth, extraWidth;
             if (this._columnWidths) {
-                if (child.span == -1)
-                    availWidth = box.x2 - x;
-                else {
+                if (child.span == -1) {
+                    if (direction == St.TextDirection.LTR)
+                        availWidth = box.x2 - x;
+                    else
+                        availWidth = x - box.x1;
+                } else {
                     availWidth = 0;
                     for (let j = 0; j < child.span; j++)
                         availWidth += this._columnWidths[col++];
@@ -257,18 +279,36 @@ PopupBaseMenuItem.prototype = {
                 extraWidth = 0;
             }
 
-            if (child.expand) {
-                childBox.x1 = x;
-                childBox.x2 = x + availWidth;
-            } else if (child.align === St.Align.CENTER) {
-                childBox.x1 = x + Math.round(extraWidth / 2);
-                childBox.x2 = childBox.x1 + naturalWidth;
-            } else if (child.align === St.Align.END) {
-                childBox.x2 = x + availWidth;
-                childBox.x1 = childBox.x2 - naturalWidth;
+            if (direction == St.TextDirection.LTR) {
+                if (child.expand) {
+                    childBox.x1 = x;
+                    childBox.x2 = x + availWidth;
+                } else if (child.align === St.Align.CENTER) {
+                    childBox.x1 = x + Math.round(extraWidth / 2);
+                    childBox.x2 = childBox.x1 + naturalWidth;
+                } else if (child.align === St.Align.END) {
+                    childBox.x2 = x + availWidth;
+                    childBox.x1 = childBox.x2 - naturalWidth;
+                } else {
+                    childBox.x1 = x;
+                    childBox.x2 = x + naturalWidth;
+                }
             } else {
-                childBox.x1 = x;
-                childBox.x2 = x + naturalWidth;
+                if (child.expand) {
+                    childBox.x1 = x - availWidth;
+                    childBox.x2 = x;
+                } else if (child.align === St.Align.CENTER) {
+                    childBox.x1 = x - Math.round(extraWidth / 2);
+                    childBox.x2 = childBox.x1 + naturalWidth;
+                } else if (child.align === St.Align.END) {
+                    // align to the left
+                    childBox.x1 = x - availWidth;
+                    childBox.x2 = childBox.x1 + naturalWidth;
+                } else {
+                    // align to the right
+                    childBox.x2 = x;
+                    childBox.x1 = x - naturalWidth;
+                }
             }
 
             let [minHeight, naturalHeight] = child.actor.get_preferred_height(-1);
@@ -277,7 +317,10 @@ PopupBaseMenuItem.prototype = {
 
             child.actor.allocate(childBox, flags);
 
-            x += availWidth + this._spacing;
+            if (direction == St.TextDirection.LTR)
+                x += availWidth + this._spacing;
+            else
+                x -= availWidth + this._spacing;
         }
     }
 };
@@ -485,12 +528,30 @@ PopupSliderMenuItem.prototype = {
         let sliderBorderColor = themeNode.get_color('-slider-border-color');
         let sliderColor = themeNode.get_color('-slider-background-color');
 
+        let sliderActiveBorderColor = themeNode.get_color('-slider-active-border-color');
+        let sliderActiveColor = themeNode.get_color('-slider-active-background-color');
+
+        cr.setSourceRGBA (
+            sliderActiveColor.red / 255,
+            sliderActiveColor.green / 255,
+            sliderActiveColor.blue / 255,
+            sliderActiveColor.alpha / 255);
+        cr.rectangle(handleRadius, (height - sliderHeight) / 2, sliderWidth * this._value, sliderHeight);
+        cr.fillPreserve();
+        cr.setSourceRGBA (
+            sliderActiveBorderColor.red / 255,
+            sliderActiveBorderColor.green / 255,
+            sliderActiveBorderColor.blue / 255,
+            sliderActiveBorderColor.alpha / 255);
+        cr.setLineWidth(sliderBorderWidth);
+        cr.stroke();
+
         cr.setSourceRGBA (
             sliderColor.red / 255,
             sliderColor.green / 255,
             sliderColor.blue / 255,
             sliderColor.alpha / 255);
-        cr.rectangle(handleRadius, (height - sliderHeight) / 2, sliderWidth, sliderHeight);
+        cr.rectangle(handleRadius + sliderWidth * this._value, (height - sliderHeight) / 2, sliderWidth * (1 - this._value), sliderHeight);
         cr.fillPreserve();
         cr.setSourceRGBA (
             sliderBorderColor.red / 255,
@@ -706,6 +767,11 @@ PopupMenuBase.prototype = {
         }
 
         this.isOpen = false;
+
+        // If set, we don't send events (including crossing events) to the source actor
+        // for the menu which causes its prelight state to freeze
+        this.blockSourceEvents = false;
+
         this._activeMenuItem = null;
     },
 
@@ -806,6 +872,8 @@ PopupMenuBase.prototype = {
         let columnWidths = [];
         let items = this.box.get_children();
         for (let i = 0; i < items.length; i++) {
+            if (!items[i].visible)
+                continue;
             if (items[i]._delegate instanceof PopupBaseMenuItem || items[i]._delegate instanceof PopupMenuBase) {
                 let itemColumnWidths = items[i]._delegate.getColumnWidths();
                 for (let j = 0; j < itemColumnWidths.length; j++) {
@@ -1311,14 +1379,20 @@ PopupMenuManager.prototype = {
         return this._activeMenuContains(event.get_source());
     },
 
-    _eventIsOnAnyMenuSource: function(event) {
+    _shouldBlockEvent: function(event) {
         let src = event.get_source();
+
+        if (this._activeMenu != null && this._activeMenu.actor.contains(src))
+            return false;
+
         for (let i = 0; i < this._menus.length; i++) {
             let menu = this._menus[i].menu;
-            if (menu.sourceActor && menu.sourceActor.contains(src))
-                return true;
+            if (menu.sourceActor && !menu.blockSourceEvents && menu.sourceActor.contains(src)) {
+                return false;
+            }
         }
-        return false;
+
+        return true;
     },
 
     _findMenu: function(item) {
@@ -1350,7 +1424,7 @@ PopupMenuManager.prototype = {
         } else if (eventType == Clutter.EventType.BUTTON_PRESS && !activeMenuContains) {
             this._closeMenu();
             return true;
-        } else if (activeMenuContains || this._eventIsOnAnyMenuSource(event)) {
+        } else if (!this._shouldBlockEvent(event)) {
             return false;
         }
 
