@@ -5,14 +5,16 @@ const Lang = imports.lang;
 const Gettext = imports.gettext.domain('gnome-shell');
 const _ = Gettext.gettext;
 const Gtk = imports.gi.Gtk;
+const Meta = imports.gi.Meta;
 const St = imports.gi.St;
 
 const DND = imports.ui.dnd;
 const IconGrid = imports.ui.iconGrid;
 const Main = imports.ui.main;
+const Overview = imports.ui.overview;
 const Search = imports.ui.search;
 
-const MAX_SEARCH_RESULTS_ROWS = 2;
+const MAX_SEARCH_RESULTS_ROWS = 1;
 
 
 function SearchResult(provider, metaInfo, terms) {
@@ -23,10 +25,10 @@ SearchResult.prototype = {
     _init: function(provider, metaInfo, terms) {
         this.provider = provider;
         this.metaInfo = metaInfo;
-        this.actor = new St.Clickable({ style_class: 'search-result',
-                                        reactive: true,
-                                        x_align: St.Align.START,
-                                        y_fill: true });
+        this.actor = new St.Button({ style_class: 'search-result',
+                                     reactive: true,
+                                     x_align: St.Align.START,
+                                     y_fill: true });
         this.actor._delegate = this;
 
         let content = provider.createResultActor(metaInfo, terms);
@@ -35,9 +37,7 @@ SearchResult.prototype = {
                                    reactive: true,
                                    track_hover: true });
             let icon = new IconGrid.BaseIcon(this.metaInfo['name'],
-                                             { createIcon: Lang.bind(this, function(size) {
-                                                 return this.metaInfo['icon'];
-                                             })});
+                                             { createIcon: this.metaInfo['createIcon'] });
             content.set_child(icon.actor);
         }
         this._content = content;
@@ -50,6 +50,10 @@ SearchResult.prototype = {
                           Lang.bind(this, function() {
                               Main.overview.beginItemDrag(this);
                           }));
+        draggable.connect('drag-cancelled',
+                          Lang.bind(this, function() {
+                              Main.overview.cancelledItemDrag(this);
+                          }));
         draggable.connect('drag-end',
                           Lang.bind(this, function() {
                               Main.overview.endItemDrag(this);
@@ -68,7 +72,7 @@ SearchResult.prototype = {
         Main.overview.toggle();
     },
 
-    _onResultClicked: function(actor, event) {
+    _onResultClicked: function(actor) {
         this.activate();
     },
 
@@ -80,11 +84,11 @@ SearchResult.prototype = {
         return new Clutter.Clone({ source: this.metaInfo['icon'] });
     },
 
-    shellWorkspaceLaunch: function() {
+    shellWorkspaceLaunch: function(params) {
         if (this.provider.dragActivateResult)
-            this.provider.dragActivateResult(this.metaInfo.id);
+            this.provider.dragActivateResult(this.metaInfo.id, params);
         else
-            this.provider.activateResult(this.metaInfo.id);
+            this.provider.activateResult(this.metaInfo.id, params);
     }
 };
 
@@ -101,8 +105,30 @@ GridSearchResults.prototype = {
         this._grid = new IconGrid.IconGrid({ rowLimit: MAX_SEARCH_RESULTS_ROWS,
                                              xAlign: St.Align.START });
         this.actor = new St.Bin({ x_align: St.Align.START });
+
         this.actor.set_child(this._grid.actor);
         this.selectionIndex = -1;
+        this._width = 0;
+        this.actor.connect('notify::width', Lang.bind(this, function() {
+            this._width = this.actor.width;
+            Meta.later_add(Meta.LaterType.BEFORE_REDRAW, Lang.bind(this, function() {
+                this._tryAddResults();
+            }));
+        }));
+        this._notDisplayedResult = [];
+        this._terms = [];
+    },
+
+    _tryAddResults: function() {
+        let canDisplay = this._grid.childrenInRow(this._width) * MAX_SEARCH_RESULTS_ROWS
+                         - this._grid.visibleItemsCount();
+
+        for (let i = Math.min(this._notDisplayedResult.length, canDisplay); i > 0; i--) {
+            let result = this._notDisplayedResult.shift();
+            let meta = this.provider.getResultMeta(result);
+            let display = new SearchResult(this.provider, meta, this._terms);
+            this._grid.addItem(display.actor);
+        }
     },
 
     getVisibleResultCount: function() {
@@ -110,15 +136,15 @@ GridSearchResults.prototype = {
     },
 
     renderResults: function(results, terms) {
-        for (let i = 0; i < results.length; i++) {
-            let result = results[i];
-            let meta = this.provider.getResultMeta(result);
-            let display = new SearchResult(this.provider, meta, terms);
-            this._grid.addItem(display.actor);
-        }
+        // copy the lists
+        this._notDisplayedResult = results.slice(0);
+        this._terms = terms.slice(0);
+        this._tryAddResults();
     },
 
     clear: function () {
+        this._terms = [];
+        this._notDisplayedResult = [];
         this._grid.removeAll();
         this.selectionIndex = -1;
     },
@@ -149,28 +175,41 @@ GridSearchResults.prototype = {
 };
 
 
-function SearchResults(searchSystem) {
-    this._init(searchSystem);
+function SearchResults(searchSystem, openSearchSystem) {
+    this._init(searchSystem, openSearchSystem);
 }
 
 SearchResults.prototype = {
-    _init: function(searchSystem) {
+    _init: function(searchSystem, openSearchSystem) {
         this._searchSystem = searchSystem;
+        this._openSearchSystem = openSearchSystem;
+
+        this.actor = new St.BoxLayout({ name: 'searchResults',
+                                        vertical: true });
 
-        this.actor = new St.Bin({ name: 'searchResults',
-                                  y_align: St.Align.START,
-                                  x_align: St.Align.START,
-                                  x_fill: true });
         this._content = new St.BoxLayout({ name: 'searchResultsContent',
                                            vertical: true });
 
         let scrollView = new St.ScrollView({ x_fill: true,
                                              y_fill: false,
-                                             vshadows: true });
+                                             vfade: true });
         scrollView.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
         scrollView.add_actor(this._content);
 
-        this.actor.set_child(scrollView);
+        this.actor.add(scrollView, { x_fill: true,
+                                     y_fill: false,
+                                     expand: true,
+                                     x_align: St.Align.START,
+                                     y_align: St.Align.START });
+        this.actor.connect('notify::mapped', Lang.bind(this,
+            function() {
+                if (!this.actor.mapped)
+                    return;
+
+                let adjustment = scrollView.vscroll.adjustment;
+                let direction = Overview.SwipeScrollDirection.VERTICAL;
+                Main.overview.setScrollAdjustment(adjustment, direction);
+            }));
 
         this._statusText = new St.Label({ style_class: 'search-statustext' });
         this._content.add(this._statusText);
@@ -179,23 +218,59 @@ SearchResults.prototype = {
         this._providerMeta = [];
         for (let i = 0; i < this._providers.length; i++)
             this.createProviderMeta(this._providers[i]);
+
+        this._searchProvidersBox = new St.BoxLayout({ style_class: 'search-providers-box' });
+        this.actor.add(this._searchProvidersBox);
+
+        this._openSearchProviders = [];
+        this._openSearchSystem.connect('changed', Lang.bind(this, this._updateOpenSearchProviderButtons));
+        this._updateOpenSearchProviderButtons();
+    },
+
+    _updateOpenSearchProviderButtons: function() {
+        this._selectedOpenSearchButton = -1;
+        for (let i = 0; i < this._openSearchProviders.length; i++)
+            this._openSearchProviders[i].actor.destroy();
+        this._openSearchProviders = this._openSearchSystem.getProviders();
+        for (let i = 0; i < this._openSearchProviders.length; i++)
+            this._createOpenSearchProviderButton(this._openSearchProviders[i]);
+    },
+
+    _updateOpenSearchButtonState: function() {
+         for (let i = 0; i < this._openSearchProviders.length; i++) {
+             if (i == this._selectedOpenSearchButton)
+                 this._openSearchProviders[i].actor.add_style_pseudo_class('selected');
+             else
+                 this._openSearchProviders[i].actor.remove_style_pseudo_class('selected');
+         }
+    },
+
+    _createOpenSearchProviderButton: function(provider) {
+        let button = new St.Button({ style_class: 'dash-search-button',
+                                     reactive: true,
+                                     x_fill: true,
+                                     y_align: St.Align.MIDDLE });
+        let bin = new St.Bin({ x_fill: false,
+                               x_align:St.Align.MIDDLE });
+        button.connect('clicked', Lang.bind(this, function() {
+            this._openSearchSystem.activateResult(provider.id);
+        }));
+        let title = new St.Label({ text: provider.name,
+                                   style_class: 'dash-search-button-label' });
+
+        bin.set_child(title);
+        button.set_child(bin);
+        provider.actor = button;
+
+        this._searchProvidersBox.add(button);
     },
 
     createProviderMeta: function(provider) {
         let providerBox = new St.BoxLayout({ style_class: 'search-section',
                                              vertical: true });
-        let titleButton = new St.Button({ style_class: 'search-section-header',
-                                          reactive: true,
-                                          x_fill: true,
-                                          y_fill: true });
-        titleButton.connect('clicked', Lang.bind(this, function () { this._onHeaderClicked(provider); }));
-        providerBox.add(titleButton);
-        let titleBox = new St.BoxLayout();
-        titleButton.set_child(titleBox);
-        let title = new St.Label({ text: provider.title });
-        let count = new St.Label();
-        titleBox.add(title, { expand: true });
-        titleBox.add(count);
+        let title = new St.Label({ style_class: 'search-section-header',
+                                   text: provider.title });
+        providerBox.add(title);
 
         let resultDisplayBin = new St.Bin({ style_class: 'search-section-results',
                                             x_fill: true,
@@ -208,8 +283,7 @@ SearchResults.prototype = {
         resultDisplayBin.set_child(resultDisplay.actor);
 
         this._providerMeta.push({ actor: providerBox,
-                                  resultDisplay: resultDisplay,
-                                  count: count });
+                                  resultDisplay: resultDisplay });
         this._content.add(providerBox);
     },
 
@@ -227,6 +301,8 @@ SearchResults.prototype = {
         this._searchSystem.reset();
         this._statusText.hide();
         this._clearDisplay();
+        this._selectedOpenSearchButton = -1;
+        this._updateOpenSearchButtonState();
     },
 
     startingSearch: function() {
@@ -247,30 +323,28 @@ SearchResults.prototype = {
         if (results.length == 0) {
             this._statusText.set_text(_("No matching results."));
             this._statusText.show();
-            return true;
         } else {
+            this._selectedOpenSearchButton = -1;
+            this._updateOpenSearchButtonState();
             this._statusText.hide();
         }
 
         let terms = this._searchSystem.getTerms();
+        this._openSearchSystem.setSearchTerms(terms);
 
         for (let i = 0; i < results.length; i++) {
             let [provider, providerResults] = results[i];
             let meta = this._metaForProvider(provider);
             meta.actor.show();
             meta.resultDisplay.renderResults(providerResults, terms);
-            meta.count.set_text('' + providerResults.length);
         }
 
-        this.selectDown(false);
+        if (this._selectedOpenSearchButton == -1)
+            this.selectDown(false);
 
         return true;
     },
 
-    _onHeaderClicked: function(provider) {
-        provider.expandSearch(this._searchSystem.getTerms());
-    },
-
     _modifyActorSelection: function(resultDisplay, up) {
         let success;
         let index = resultDisplay.getSelectionIndex();
@@ -284,16 +358,26 @@ SearchResults.prototype = {
     },
 
     selectUp: function(recursing) {
-        for (let i = this._selectedProvider; i >= 0; i--) {
-            let meta = this._providerMeta[i];
-            if (!meta.actor.visible)
-                continue;
-            let success = this._modifyActorSelection(meta.resultDisplay, true);
-            if (success) {
-                this._selectedProvider = i;
-                return;
+        if (this._selectedOpenSearchButton == -1) {
+            for (let i = this._selectedProvider; i >= 0; i--) {
+                let meta = this._providerMeta[i];
+                if (!meta.actor.visible)
+                    continue;
+                let success = this._modifyActorSelection(meta.resultDisplay, true);
+                if (success) {
+                    this._selectedProvider = i;
+                    return;
+                }
             }
         }
+
+        if (this._selectedOpenSearchButton == -1)
+            this._selectedOpenSearchButton = this._openSearchProviders.length;
+        this._selectedOpenSearchButton--;
+        this._updateOpenSearchButtonState();
+        if (this._selectedOpenSearchButton >= 0)
+            return;
+
         if (this._providerMeta.length > 0 && !recursing) {
             this._selectedProvider = this._providerMeta.length - 1;
             this.selectUp(true);
@@ -302,18 +386,30 @@ SearchResults.prototype = {
 
     selectDown: function(recursing) {
         let current = this._selectedProvider;
-        if (current == -1)
-            current = 0;
-        for (let i = current; i < this._providerMeta.length; i++) {
-            let meta = this._providerMeta[i];
-            if (!meta.actor.visible)
-                continue;
-            let success = this._modifyActorSelection(meta.resultDisplay, false);
-            if (success) {
-                this._selectedProvider = i;
-                return;
+        if (this._selectedOpenSearchButton == -1) {
+            if (current == -1)
+                current = 0;
+            for (let i = current; i < this._providerMeta.length; i++) {
+                let meta = this._providerMeta[i];
+                if (!meta.actor.visible)
+                    continue;
+                 let success = this._modifyActorSelection(meta.resultDisplay, false);
+                 if (success) {
+                    this._selectedProvider = i;
+                    return;
+                 }
             }
         }
+        this._selectedOpenSearchButton++;
+
+        if (this._selectedOpenSearchButton < this._openSearchProviders.length) {
+            this._updateOpenSearchButtonState();
+            return;
+        }
+
+        this._selectedOpenSearchButton = -1;
+        this._updateOpenSearchButtonState();
+
         if (this._providerMeta.length > 0 && !recursing) {
             this._selectedProvider = 0;
             this.selectDown(true);
@@ -321,6 +417,13 @@ SearchResults.prototype = {
     },
 
     activateSelected: function() {
+        if (this._selectedOpenSearchButton != -1) {
+            let provider = this._openSearchProviders[this._selectedOpenSearchButton];
+            this._openSearchSystem.activateResult(provider.id);
+            Main.overview.hide();
+            return;
+        }
+
         let current = this._selectedProvider;
         if (current < 0)
             return;
