@@ -1213,3 +1213,14 @@ st_box_layout_insert_actor (StBoxLayout  *self,
   clutter_container_add_actor((ClutterContainer*) self, actor);
   st_container_move_child (ST_CONTAINER (self), actor, pos);
 }
+
+void
+st_box_layout_insert_before (StBoxLayout  *self,
+                             ClutterActor *actor,
+                             ClutterActor *sibling)
+{
+  g_return_if_fail (ST_IS_BOX_LAYOUT (self));
+
+  clutter_container_add_actor(CLUTTER_CONTAINER (self), actor);
+  st_container_move_before (ST_CONTAINER (self), actor, sibling);
+}
